import aiohttp
from .get_data import get_data
from ..DomoAuth import DomoFullAuth
from ...utils.ResponseGetData import ResponseGetData


def generate_body_create_group(group_name: str,
                               group_type: str,
                               description: str) -> dict:
    body = {"name": group_name, "type": group_type or 'open',
            "description": description or ''}

    return body


async def create_group(full_auth: DomoFullAuth,
                       group_name: str = None,
                       group_type: str = None,
                       description: str = None,
                       log_results: bool = False, debug: bool = False) -> ResponseGetData:
    # body : {"name": "GROUP_NAME", "type": "open", "description": ""}

    body = generate_body_create_group(
        group_name=group_name, group_type=group_type, description=description)
    print({'create_group_body': body})

    url = f'https://{full_auth.domo_instance}.domo.com/api/content/v2/groups/'

    if log_results:
        print(f'Creating Group: {body.get("name")}- at {url}')

    res = await get_data(
        auth=full_auth,
        url=url,
        method='POST',
        body=body,
        debug=debug
    )

    return res


async def get_all_groups(full_auth: DomoFullAuth,
                         log_results: bool = False,
                         debug: bool = False,
                         session: aiohttp.ClientSession = None) -> ResponseGetData:
    if debug:
        print(full_auth)

    url = f'https://{full_auth.domo_instance}.domo.com/api/content/v2/groups/grouplist'

    if debug:
        print(full_auth, url)

    if log_results:
        print(f'Getting groups from - {url}')

    res = await get_data(url=url, method='GET', auth=full_auth, session=session)

    return res


async def search_groups_by_name(full_auth: DomoFullAuth,
                                search_name: str,
                                debug: bool = False, log_results: bool = False) -> ResponseGetData:
    url = f'https://{full_auth.domo_instance}.domo.com/api/content/v2/groups/grouplist?ascending=true&search={search_name}&sort=name '

    res = await get_data(
        auth=full_auth,
        url=url,
        method='GET',
        debug=debug,
        log_results=log_results
    )
    return res


def generate_body_update_group_membership(group_id: str,
                                          add_user_arr: list[str] = None,
                                          remove_user_arr: list[str] = None,
                                          add_owner_user_arr: list[str] = None,
                                          remove_owner_user_arr: list[str] = None) -> list[dict]:
    body = {"groupId": int(group_id)}
    if add_owner_user_arr:
        body.update({"addOwners": [{"type": "USER", "id": str(
            userId)} for userId in add_owner_user_arr]})

    if remove_owner_user_arr:
        body.update({"removeOwners": [{"type": "USER", "id": str(
            userId)} for userId in remove_owner_user_arr]})

    if remove_user_arr:
        body.update({"removeMembers": [
                    {"type": "USER", "id": str(userId)} for userId in remove_user_arr]})
    if add_user_arr:
        body.update(
            {"addMembers": [{"type": "USER", "id": str(userId)} for userId in add_user_arr]})

    return [body]


async def update_group_membership(full_auth: DomoFullAuth,
                                  body: dict,
                                  log_results: bool = False, debug: bool = False) -> ResponseGetData:
    # body = [{
    #     "groupId":"GROUP_ID",
    #     "removeMembers": [{"type":"USER","id":"USER_ID"}],
    #     "addMembers"   : [{"type":"USER","id":"USER_ID"}]
    # }]

    url = f'https://{full_auth.domo_instance}.domo.com/api/content/v2/groups/access'

    if debug:
        print(url, body)

    res = await get_data(
        auth=full_auth,
        url=url,
        method='PUT',
        body=body,
        debug=debug
    )

    return res

async def get_group_by_id(full_auth:DomoFullAuth, group_id:id, debug:bool = False) -> ResponseGetData:
    url = f'https://{full_auth.domo_instance}.domo.com/api/content/v2/groups/{group_id}'

    if debug:
        print(url)

    res = await get_data(
        auth=full_auth,
        url=url,
        method='GET',
        debug=debug
    )

    return res


import aiohttp
from dataclasses import dataclass

from .DomoAuth import DomoFullAuth
from .DomoDataset import DomoDataset
from .routes import bootstrap_routes
from ..utils.DictDot import DictDot


@dataclass
class DomoBootstrapFeature:
    id: int
    name: str
    label: str
    type: str
    purchased: bool
    enabled: bool

    @classmethod
    def create_from_json_bootstrap(cls, json_obj: dict):
        dd = DictDot(json_obj)

        bsf = cls(
            id=dd.id,
            name=dd.name,
            label=dd.label,
            type=dd.type,
            purchased=dd.purchased,
            enabled=dd.enabled
        )
        return bsf


class DomoBootstrap:

    @classmethod
    async def get_all(cls, full_auth: DomoFullAuth, debug: bool = False):
        return await bootstrap_routes.bsr_all(full_auth=full_auth, debug=debug)

    @classmethod
    async def get_pages(cls, full_auth: DomoFullAuth, debug: bool = False):
        return await bootstrap_routes.bsr_pages(full_auth=full_auth, debug=debug)

    @classmethod
    async def get_features(cls, full_auth: DomoFullAuth,
                           session: aiohttp.ClientSession = None, debug: bool = False):
        is_close_session= False
        if not session:
            session = aiohttp.ClientSession()
            is_close_session= True
            
        json_list = await bootstrap_routes.bsr_features(full_auth=full_auth, session=session, debug=debug)

        if is_close_session:
            await session.close()
            
        feature_list = [DomoBootstrapFeature.create_from_json_bootstrap(
            json_obj) for json_obj in json_list]
        
        return feature_list



from dataclasses import dataclass, field

from Library.utils.DictDot import DictDot
from Library.DomoClasses.DomoAuth import DomoFullAuth
from Library.DomoClasses.routes import grant_routes


@dataclass
class DomoGrant:
    id: str
    display_group: str
    depends_on: [str]
    title: str
    description: str = None
    role_membership: [str] = None

    @classmethod
    def _from_json(cls, obj):
        dd = DictDot(obj)

        return cls(id=dd.authority,
                   display_group=dd.authorityUIGroup,
                   depends_on=dd.dependsOnAuthorities,
                   title=dd.title,
                   description=dd.description,
                   role_membership=[str(role) for role in dd.roleIds])
